@@ -4,6 +4,7 @@ from typing import Any, Dict, List, Optional
 
 import numpy as np
 import pandas as pd
+from pandas.api.types import is_numeric_dtype
 
 from .ensemble_set_model import EnsembleSetModel
 from .parameter_model import ParametersModel
@@ -22,6 +23,7 @@ class InplaceVolumesModel:
         "REGION",
         "FACIES",
         "LICENSE",
+        "SENSNAME_CASE",
         "SENSNAME",
         "SENSCASE",
         "SENSTYPE",
@@ -59,7 +61,7 @@ class InplaceVolumesModel:
                     volumes_table, selectors
                 )
 
-            # stack dataframe on fluid zone and add fluid as column istead of a column suffix
+            # stack dataframe on fluid zone and add fluid as column instead of a column suffix
             dfs = []
             for fluid in ["OIL", "GAS", "WATER"]:
                 fluid_columns = [
@@ -100,6 +102,9 @@ class InplaceVolumesModel:
 
         self._dataframe.sort_values(by=["ENSEMBLE", "REAL"], inplace=True)
 
+        # ensure ensemble column consists of strings
+        self._dataframe["ENSEMBLE"] = self._dataframe["ENSEMBLE"].astype(str)
+
         # compute and set property columns
         self._set_initial_property_columns()
         self._dataframe = self.compute_property_columns(self._dataframe)
@@ -136,6 +141,21 @@ class InplaceVolumesModel:
     def ensembles(self) -> List[str]:
         return list(self._dataframe["ENSEMBLE"].unique())
 
+    @property
+    def ensemble_sensitivities(self) -> Dict[str, list]:
+        return {
+            ens: (
+                list(
+                    self._dataframe.loc[
+                        self._dataframe["ENSEMBLE"] == ens, "SENSNAME_CASE"
+                    ].unique()
+                )
+                if "SENSNAME_CASE" in self._dataframe
+                else []
+            )
+            for ens in self.ensembles
+        }
+
     @property
     def property_columns(self) -> List[str]:
         return self._property_columns
@@ -145,7 +165,9 @@ class InplaceVolumesModel:
         return [
             x
             for x in self._dataframe
-            if x not in self.selectors and x not in self.property_columns
+            if x not in self.selectors
+            and x not in self.property_columns
+            and is_numeric_dtype(self._dataframe[x])
         ]
 
     @property
@@ -164,6 +186,14 @@ class InplaceVolumesModel:
     def responses(self) -> List[str]:
         return self.volume_columns + self.property_columns
 
+    @property
+    def hc_responses(self) -> List[str]:
+        return [
+            x
+            for x in ["STOIIP", "GIIP", "ASSOCIATEDGAS", "ASSOCIATEDOIL"]
+            if x in self.volume_columns
+        ]
+
     @property
     def parameters(self) -> List[str]:
         return self.pmodel.parameters
@@ -202,11 +232,15 @@ class InplaceVolumesModel:
         available volume columns"""
         self._property_columns = []
 
-        if all(col in self._dataframe for col in ["NET", "BULK"]):
+        if all(
+            col in self._dataframe for col in ["NET", "BULK"]
+        ) and not self._dataframe["NET"].equals(self._dataframe["BULK"]):
             self._property_columns.append("NTG")
         if all(col in self._dataframe for col in ["BULK", "PORV"]):
             self._property_columns.append("PORO")
-        if all(col in self._dataframe for col in ["NET", "PORV"]):
+        if all(
+            col in self._dataframe for col in ["NET", "PORV"]
+        ) and not self._dataframe["NET"].equals(self._dataframe["BULK"]):
             self._property_columns.append("PORO_NET")
         if all(col in self._dataframe for col in ["HCPV", "PORV"]):
             self._property_columns.append("SW")
@@ -227,9 +261,9 @@ class InplaceVolumesModel:
         if "NTG" in properties:
             dframe["NTG"] = dframe["NET"] / dframe["BULK"]
         if "PORO" in properties:
-            if "NET" in dframe.columns:
-                dframe["PORO_NET"] = dframe["PORV"] / dframe["NET"]
             dframe["PORO"] = dframe["PORV"] / dframe["BULK"]
+        if "PORO_NET" in properties:
+            dframe["PORO_NET"] = dframe["PORV"] / dframe["NET"]
         if "SW" in properties:
             dframe["SW"] = 1 - (dframe["HCPV"] / dframe["PORV"])
         if "BO" in properties:
@@ -280,16 +314,16 @@ class InplaceVolumesModel:
         dframe = self.compute_property_columns(dframe, properties)
         if "FLUID_ZONE" not in groups:
             if not filters.get("FLUID_ZONE") == ["oil"]:
-                dframe["BO"] = "NA"
+                dframe["BO"] = np.nan
             if not filters.get("FLUID_ZONE") == ["gas"]:
-                dframe["BG"] = "NA"
+                dframe["BG"] = np.nan
         return dframe
 
 
 def filter_df(dframe: pd.DataFrame, filters: dict) -> pd.DataFrame:
     """
     Filter dataframe using dictionary with form
-    'column_name': [list ov values to keep]
+    'column_name': [list of values to keep]
     """
     for filt, values in filters.items():
         dframe = dframe.loc[dframe[filt].isin(values)]
