@@ -1,23 +1,22 @@
-from typing import Tuple, Callable
+from typing import Callable, Tuple
 
 import pandas as pd
-import dash
-from dash.dependencies import Input, Output, ALL
-import dash_html_components as html
 import plotly.graph_objects as go
+from dash import ALL, Dash, Input, Output, html
+from dash.exceptions import PreventUpdate
 
 from webviz_subsurface._models import SurfaceLeafletModel
-from ..views.property_delta_view import table_view, surface_views
-from ..utils.surface import surface_from_zone_prop
 
 from ..models import PropertyStatisticsModel
+from ..utils.surface import surface_from_zone_prop
+from ..views.property_delta_view import surface_views, table_view
 
 
 def property_delta_controller(
     get_uuid: Callable,
     property_model: PropertyStatisticsModel,
     surface_table: pd.DataFrame,
-    app: dash.Dash,
+    app: Dash,
 ) -> None:
     @app.callback(
         Output(get_uuid("delta-bar-graph"), "figure"),
@@ -45,6 +44,10 @@ def property_delta_controller(
         selectors: list,
         clickdata: dict,
     ) -> Tuple[go.Figure, html.Div]:
+        # Prevent update if some filters are empty
+        if not all(filt for filt in selectors):
+            raise PreventUpdate
+
         # Make bar chart
         bars = property_model.make_delta_bars(
             prop=prop,
